package org.dbxml.examples;

/*
 * dbXML Core - Example Code
 *
 * Copyright (c) 1999-2001 The dbXML Group
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy 
 * of this software and associated documentation files (the "Software"), to 
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or 
 * sell copies of the Software, and to permit persons to whom the Software is 
 * furnished to do so, subject to the following conditions:
 *
 *    The above copyright notice and this permission notice shall be included 
 *       in all copies of substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
 * IN THE SOFTWARE.
 *
 * $Id: AddDocument.java,v 1.2 2001/09/06 18:32:56 kstaken Exp $
 */

import org.xmldb.api.base.*;
import org.xmldb.api.modules.*;
import org.xmldb.api.*;

import java.io.*;

/**
 * Simple XML:DB API Example to insert a new document into the database.
 */
public class AddDocument {
   public static void main(String[] args) throws Exception {
      Collection col = null;
      try {           
         String driver = "org.dbxml.client.xmldb.DatabaseImpl";
         Class c = Class.forName(driver);
         
         Database database = (Database) c.newInstance();
         DatabaseManager.registerDatabase(database);
         col =
            DatabaseManager.getCollection("xmldb:dbxml:///db/addressbook");
   
         String data = readFileFromDisk(args[0]);
         
         XMLResource document = (XMLResource) col.createResource(null, 
            "XMLResource");
         document.setContent(data);
         col.storeResource(document);
         System.out.println("Document " + args[0] + " inserted as " + 
            document.getId());
      }
      catch (XMLDBException e) {
         System.err.println("XML:DB Exception occured " + e.errorCode + " " + 
            e.getMessage());
      }
      finally {
         if (col != null) {
            col.close();
         }
      }
   }
   
   public static String readFileFromDisk(String fileName) throws Exception {                                     
      File file = new File(fileName);
      FileInputStream insr = new FileInputStream(file);
                  
      byte[] fileBuffer = new byte[(int)file.length()];
   
      insr.read(fileBuffer);
      insr.close();
      
      return new String(fileBuffer);   
   }
}
