package org.dbxml.core.indexer;

/*
 * dbXML License, Version 1.0
 *
 *
 * Copyright (c) 1999-2001 The dbXML Group, L.L.C.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by
 *        The dbXML Group (http://www.dbxml.com/)."
 *    Alternately, this acknowledgment may appear in the software
 *    itself, if and wherever such third-party acknowledgments normally
 *    appear.
 *
 * 4. The names "dbXML" and "The dbXML Group" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    info@dbxml.com.
 *
 * 5. Products derived from this software may not be called "dbXML",
 *    nor may "dbXML" appear in their name, without prior written
 *    permission of The dbXML Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE DBXML GROUP OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: IndexMatch.java,v 1.8 2001/06/29 22:04:25 tom Exp $
 */

import org.dbxml.core.data.*;
import org.dbxml.core.filer.*;
import java.io.*;

/**
 * IndexMatch is used for representing match information.  IndexMatches
 * are the basic unit of information between Indexers and the IndexManager
 * for performing queries and query optimization.
 */

public final class IndexMatch implements Comparable {
   private Key key;    // The Document Key
   private int pos;    // Offset into the stream
   private int len;    // Length of substream
   private short elem; // Element ID (for wildcards)
   private short attr; // Attribute ID (if any)
   
   public IndexMatch() {
      key = null;
      pos = -1;
      len = -1;
      elem = -1;
      attr = -1;
   }

   public IndexMatch(Key key, int pos, int len, short elem, short attr) {
      this.key = key;
      this.pos = pos;
      this.len = len;
      this.elem = elem;
      this.attr = attr;
   }

   public IndexMatch(Key key, int pos, int len) {
      this.key = key;
      this.pos = pos;
      this.len = len;
      elem = -1;
      attr = -1;
   }
   
   public IndexMatch(Key key, IndexPattern pattern) {
      this.key = key;
      pos = -1;
      len = -1;
      elem = pattern.getElementID();
      attr = pattern.getAttributeID();
   }
   
   /**
    * getKey returns the Document Key for the IndexMatch.
    *
    * @return The Key
    */
   public final Key getKey() {
      return key;
   }

   /**
    * getPosition returns the Document position for the Match.
    *
    * @return The Document position
    */
   public final int getPosition() {
      return pos;
   }

   /**
    * getLength returns the Node length for the Match.
    *
    * @return The Node length
    */
   public final int getLength() {
      return len;
   }

   /**
    * getElement returns the Element symbol ID for the Match.
    *
    * @return The Element Symbol ID
    */
   public final short getElement() {
      return elem;
   }

   /**
    * getAttribute returns the Attribute symbol ID for the Match.
    *
    * @return The Attribute Symbol ID
    */
   public final short getAttribute() {
      return attr;
   }
      
   public int compareTo(Object obj) {
      return key.compareTo(obj);
   }
   
   public String toString() {
      return key.toString();
   }
}

