package org.dbxml.core.objects;

/*
 * dbXML License, Version 1.0
 *
 *
 * Copyright (c) 1999-2001 The dbXML Group, L.L.C.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by
 *        The dbXML Group (http://www.dbxml.com/)."
 *    Alternately, this acknowledgment may appear in the software
 *    itself, if and wherever such third-party acknowledgments normally
 *    appear.
 *
 * 4. The names "dbXML" and "The dbXML Group" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    info@dbxml.com.
 *
 * 5. Products derived from this software may not be called "dbXML",
 *    nor may "dbXML" appear in their name, without prior written
 *    permission of The dbXML Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE DBXML GROUP OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: Types.java,v 1.10 2001/05/23 00:35:12 tom Exp $
 */

import org.w3c.dom.*;
import org.dbxml.core.*;

/**
 * Types defines identifying values for the types that dbXML Reflectable
 * methods support.
 */

public final class Types {
   public final static int EMPTY = -1;
   public final static int UNKNOWN = -1;
   public final static int VOID = 0;
   public final static int BOOLEAN = 1;
   public final static int BYTE = 2;
   public final static int CHAR = 3;
   public final static int SHORT = 4;
   public final static int INT = 5;
   public final static int LONG = 6;
   public final static int FLOAT = 7;
   public final static int DOUBLE = 8;
   public final static int OBJECT = 10;
   public final static int STRING = 11;
   public final static int ELEMENT = 12;
   public final static int DOCUMENT = 13;
   public final static int DOCUMENTFRAGMENT = 14;
   public final static int ARGS = 15;
   public final static int VARIANT = 20;

   private Types() {
   }

   /**
    * TypeOf inspects the Class, and returns the type value for that Class
    * or UNKNOWN, if the Class is not an acceptable signature type.
    *
    * @param c The Class to check
    * @return The type value
    */
   public static int typeOf(Class c) {
      if( c.isArray() )
         c = c.getComponentType();

      if ( c == void.class )
         return VOID;

      if ( c.isPrimitive() ) {
         if ( c == Boolean.TYPE )
            return BOOLEAN;
         else if ( c == Byte.TYPE )
            return BYTE;
         else if ( c == Character.TYPE )
            return CHAR;
         else if ( c  == Short.TYPE )
            return SHORT;
         else if ( c == Integer.TYPE )
            return INT;
         else if ( c == Long.TYPE )
            return LONG;
         else if ( c == Float.TYPE )
            return FLOAT;
         else if ( c == Double.TYPE )
            return DOUBLE;
      }
      else {
         if ( c == String.class )
            return STRING;
         else if ( c == Variant.class )
            return VARIANT;
         else if ( Element.class.isAssignableFrom(c) )
            return ELEMENT;
         else if ( Document.class.isAssignableFrom(c) )
            return DOCUMENT;
         else if ( DocumentFragment.class.isAssignableFrom(c) )
            return DOCUMENTFRAGMENT;
         else if ( Args.class.isAssignableFrom(c) )
            return ARGS;
      }
      return UNKNOWN;
   }

   public static int typeOf(Object object) {
      return object != null ? typeOf(object.getClass())
                            : EMPTY;
   }

   public static String typeName(int type) {
      switch ( type ) {

         case UNKNOWN          : return "Unknown";
         case VOID             : return "void";
         case BOOLEAN          : return "boolean";
         case BYTE             : return "byte";
         case CHAR             : return "char";
         case SHORT            : return "short";
         case INT              : return "int";
         case LONG             : return "long";
         case FLOAT            : return "float";
         case DOUBLE           : return "double";
         case OBJECT           : return "Object";
         case STRING           : return "String";
         case ELEMENT          : return "Element";
         case DOCUMENT         : return "Document";
         case DOCUMENTFRAGMENT : return "DocumentFragment";
         case ARGS             : return "Args";
         case VARIANT          : return "Variant";

      }
      return "Invalid";
   }
}

