package org.dbxml.core.security;

/*
 * dbXML License, Version 1.0
 *
 *
 * Copyright (c) 1999-2001 The dbXML Group, L.L.C.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by
 *        The dbXML Group (http://www.dbxml.com/)."
 *    Alternately, this acknowledgment may appear in the software
 *    itself, if and wherever such third-party acknowledgments normally
 *    appear.
 *
 * 4. The names "dbXML" and "The dbXML Group" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    info@dbxml.com.
 *
 * 5. Products derived from this software may not be called "dbXML",
 *    nor may "dbXML" appear in their name, without prior written
 *    permission of The dbXML Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE DBXML GROUP OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: DBSecurityManager.java,v 1.6 2001/05/23 00:35:28 tom Exp $
 */

 import org.dbxml.server.*;

/**
 * SecurityManager is the interface to the access control functionality of the
 * server.
 */
public interface DBSecurityManager extends Named, Configurable {
   /**
    * Determines if the provided credentials are valid for accessing the
    * specified resource.
    *
    * @param resource The resource to check access for.
    * @param requestedAccess The access level being requested.
    * @param cred The credentials to use to authenticate
    * @exception AccessDeniedException Thrown when access to the resource is
    *            denied.
    * @exception InvalidCredentialsException Thrown if the provided credentials
    *            are not valid.
    */
   void checkAccess(String resource, int requestedAccess, Credentials cred)
      throws AccessDeniedException, InvalidCredentialsException;

   /**
    * Determines if the credentials retrieved from thread local storage are
    * valid for accessing the specified resource.
    *
    * @param resource The resource to check access for.
    * @param requestedAccess The access level being requested.
    * @param cred The credentials to use to authenticate
    * @exception AccessDeniedException Thrown when access to the resource is
    *            denied.
    * @exception InvalidCredentialsException Thrown if the credentials retrieved
    *            from thread local storage are not valid.
    */
   void checkAccess(String resource, int requestedAccess)
      throws AccessDeniedException, InvalidCredentialsException;

   /**
    * Authenticates the user and retrieves a credentials object for the
    * provided username and password. If
    * an existing object can be found in the cache it will be returned otherwise
    * a new Credentials instance will be created and returned. Cached objects
    * are verified before being returned.
    *
    * @param username The username to authenticate
    * @param password The password used to authenticate the user.
    * @exception InvalidPasswordException Thrown when the password provided does
    *            not match for the username.
    * @exception UnknownUserException Thrown when the username can not be found.
    */
   Credentials authenticate(String username, String password)
      throws InvalidPasswordException, UnknownUserException,
         InvalidCredentialsException;

   /**
    * Removes any credentials associated with the current thread. Any further
    * access by the thread will result in a InvalidCredentialsException being
    * thrown.
    */
   void logout();

   /**
    * Reads the security managers configuration and prepares it for operation.
    */
   void readConfig();

   void setActive();
}
