package org.dbxml.tools.command;

/*
 * dbXML License, Version 1.0
 *
 *
 * Copyright (c) 1999-2001 The dbXML Group, L.L.C.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by
 *        The dbXML Group (http://www.dbxml.com/)."
 *    Alternately, this acknowledgment may appear in the software
 *    itself, if and wherever such third-party acknowledgments normally
 *    appear.
 *
 * 4. The names "dbXML" and "The dbXML Group" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    info@dbxml.com.
 *
 * 5. Products derived from this software may not be called "dbXML",
 *    nor may "dbXML" appear in their name, without prior written
 *    permission of The dbXML Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE DBXML GROUP OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: AddDocument.java,v 1.50 2001/09/04 11:13:48 kstaken Exp $
 */
import java.util.Hashtable;
import java.io.*;

import org.xmldb.api.base.*;
import org.xmldb.api.modules.*;
import org.xmldb.api.DatabaseManager;

import org.dbxml.client.xmldb.*;
import org.dbxml.client.xmldb.services.*;

import org.dbxml.xml.dom.*;

import org.w3c.dom.*;

/**
 * AddDocument.java is designed to let the user add Single Documents
 * and/or Multiple Documents to an existing Collection or Nested Collection
 * within the current Database.
 */

public class AddDocument extends Command {
   /**
    * TODO: Description of the Method
    *
    * @param table Description of Parameter
    * @return Description of the Returned Value
    * @exception Exception Description of Exception
    */
   public boolean execute(Hashtable table) throws Exception {

      Collection col = null;

      if (table.get("collection") == null) {
         System.out.println("ERROR : Collection and switch required");
         return false;
      }

      if (table.get("filePath") == "") {
         System.out.println("ERROR : File path required");
         return false;
      }

      try {

         // Create a collection instance
         String colstring = normalizeCollectionURI((String) table.get("collection"));
         col = DatabaseManager.getCollection(colstring);
         if (col == null) {
            System.out.println("ERROR : Collection not found!");
            return false;
         }

         // Open a new file input stream and read in the document
         File file = new File((String) table.get("filePath"));
         FileInputStream insr = new FileInputStream(file);

         byte[] fileBuffer = new byte[(int) file.length()];

         insr.read(fileBuffer);
         insr.close();

         // Create the XMLResource and store the document
         XMLResource resource =
               (XMLResource) col.createResource((String) table.get("nameOf"), 
                  "XMLResource");

         resource.setContent(new String(fileBuffer));
         col.storeResource(resource);
                           
         System.out.println("Added document " + table.get("collection") + "/" + 
            resource.getId());
         resource = null;
      }
      catch (Exception e) {
         System.out.println("ERROR : " + e.getMessage());
         if (table.get("verbose").equals("true")) {
            System.out.println("Document name = " + (String) table.get("nameOf"));
            e.printStackTrace(System.out);
         }
         return false;
      }
      finally {
         if (col != null) {
            col.close();
         }
         col = null;
      }
      
      return true;
   }

}



