package org.dbxml.xml.dom;

/*
 * dbXML License, Version 1.0
 *
 *
 * Copyright (c) 1999-2001 The dbXML Group, L.L.C.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by
 *        The dbXML Group (http://www.dbxml.com/)."
 *    Alternately, this acknowledgment may appear in the software
 *    itself, if and wherever such third-party acknowledgments normally
 *    appear.
 *
 * 4. The names "dbXML" and "The dbXML Group" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    info@dbxml.com.
 *
 * 5. Products derived from this software may not be called "dbXML",
 *    nor may "dbXML" appear in their name, without prior written
 *    permission of The dbXML Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE DBXML GROUP OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: EntityReferenceImpl.java,v 1.14 2001/08/05 07:23:17 tom Exp $
 */

import org.dbxml.xml.*;
import org.dbxml.server.util.*;

import org.w3c.dom.*;

/**
 * EntityReferenceImpl
 */

public final class EntityReferenceImpl extends NodeImpl implements EntityReference {
   private short symbolID = -1;

   public EntityReferenceImpl() {
   }
   
   public EntityReferenceImpl(NodeImpl parentNode, byte[] data, int pos, int len) {
      super(parentNode, data, pos, len);
   }

   public EntityReferenceImpl(NodeImpl parentNode, boolean dirty) {
      super(parentNode, dirty);
   }

   public EntityReferenceImpl(NodeImpl parentNode, String nodeName) {
      super(parentNode, true);
      this.nodeName = nodeName;
   }

   protected void checkLoaded() {
      if ( loaded )
         return;
      else
         loaded = true;

      try {
         if ( data != null ) {
            DocumentImpl doc = (DocumentImpl)getOwnerDocument();
            SymbolTable st = doc.getSymbols();

            ByteArrayInput bis = new ByteArrayInput(data, pos, len);
            XMLCompressedInput xci = new XMLCompressedInput(bis, st);

            byte signature = xci.readByte();
            byte entityType = (byte)(signature & 0x1F);
            switch ( entityType ) {

               case Signatures.EntUnknown:
                  nodeName = null;
                  break;

               case Signatures.EntDefined:
                  symbolID = xci.readShort();
                  nodeName = st.getName(symbolID);
                  break;

               case Signatures.EntAmp:
                  nodeName = "&amp;";
                  break;

               case Signatures.EntLt:
                  nodeName = "&lt;";
                  break;

               case Signatures.EntGt:
                  nodeName = "&gt;";
                  break;

               case Signatures.EntQuot:
                  nodeName = "&quot;";
                  break;

               case Signatures.EntApos:
                  nodeName = "&apos;";
                  break;

               case Signatures.EntUnicode:
                  nodeName = "";
                  // TODO: This
                  break;

            }
         }
      }
      catch ( Exception e ) {
         org.dbxml.Debug.printStackTrace(e);
      }
   }

   public short getNodeType() {
      return Node.ENTITY_REFERENCE_NODE;
   }
}


