
/*
 * dbXML License, Version 1.0
 *
 *
 * Copyright (c) 1999-2001 The dbXML Group, L.L.C.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by
 *        The dbXML Group (http://www.dbxml.com/)."
 *    Alternately, this acknowledgment may appear in the software
 *    itself, if and wherever such third-party acknowledgments normally
 *    appear.
 *
 * 4. The names "dbXML" and "The dbXML Group" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    info@dbxml.com.
 *
 * 5. Products derived from this software may not be called "dbXML",
 *    nor may "dbXML" appear in their name, without prior written
 *    permission of The dbXML Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE DBXML GROUP OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: DatabaseTest.java,v 1.3 2001/08/15 20:06:49 tom Exp $
 */

/*  ASSUMPTIONS
*
*  This code assumes that the collection INSTANCE_NAME/TEST_COLLECTION_NAME exists
*  as well as it's children INSTANCE_NAME/TEST_COLLECTION_CHILD1_NAME and INSTANCE_NAME/TEST_COLLECTION_CHILD2_NAME
*/

import java.io.*;

import junit.framework.*;

import org.xmldb.api.*;
import org.xmldb.api.base.*;
import org.xmldb.api.modules.*;

import org.dbxml.core.corba.db.*;
import org.dbxml.core.corba.db.Collection;
import org.dbxml.core.corba.db.DatabaseManager;

public class DatabaseTest extends CorbaAPITestCase {

   private boolean exceptioncaught = false;
   
   public DatabaseTest(String name) {
      super(name);
   }

   public static junit.framework.Test suite() {
      System.out.println("In DatabaseTest.suite() ");
      return new junit.framework.TestSuite(DatabaseTest.class);
   }

   public void testGetName() {
      try {
      
         // Test that getName() returns the correct instance name
         assertTrue( db.getName().equals(INSTANCE_NAME) );
      } catch (Exception e) {
         fail( e.getMessage() );
      }
   }

   public void testGetDatabaseManager() {

      DatabaseManager dbman = null;
      try {

         dbman = db.getDatabaseManager();
         
         // call getName() on the dbmanager, should get "db" for the root collection
         assertTrue( dbman.getName().equals(INSTANCE_NAME) );

      } catch (Exception e) {
         fail( e.getMessage() );
      } finally {
         dbman.remove();
      }
   }

   public void testListCollections() {
      try {

         // this one is tough, basicall verify that listCollections() does not return null and > 0
         assertTrue( db.listCollections() != null );
         assertTrue( db.listCollections().length > 0 );
         
      } catch (Exception e) {
         fail( e.getMessage() );
      }
   }

   public void testGetCollection() {
   
      Collection gottencol = null;
      
      try {
      
         // Retrieve an instance of our test collection
         gottencol = db.getCollection(TEST_COLLECTION_NAME);
         
         // Verify that the correct col was returned
         assertTrue( gottencol.getName().equals(TEST_COLLECTION_NAME) );
         
         // Try getting a collection with the empty string as the name
         try {
            exceptioncaught = false;
            gottencol = db.getCollection("");
         } catch(APIException e) {
            exceptioncaught=true;
            assertTrue( e.faultCode == FaultCodes.COL_COLLECTION_NOT_FOUND );
         } finally {
            assertTrue( exceptioncaught );
         }
         
         // Try getting a collection with a bogus name
         try {
            exceptioncaught = false;
            gottencol = db.getCollection("totallybogusname");
         } catch(APIException e) {
            exceptioncaught=true;
            assertTrue( e.faultCode == FaultCodes.COL_COLLECTION_NOT_FOUND );
         } finally {
            assertTrue( exceptioncaught );
         }

      } catch (Exception e) {
         fail( e.getMessage() );
      } finally {
         gottencol.remove();
      }
   }

}
